package sorting.main;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import sorting.algorithms.Algorithm;
import sorting.algorithms.AlgorithmFactory;
import sorting.model.*;
import sorting.views.*;
/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Theodore Norvell
 * @version 1.0
 */

public class Main {
    final ObservableArray array = new ObservableArray(10, 100) ;
    { array.randomize() ; }
    
    JButton randomizeButton ;
    AlgorithmFactory algorithmFactory ;
    ViewFactory viewFactory ;
    JFrame [] viewFrames ;
    JButton [] sortButtons ;
    JButton stopButton ;
    JFrame mainFrame ;
    
    

    private void cleanUp() {
		try {
			SwingUtilities.invokeAndWait( new Runnable() {
				@Override
				public void run() {
					stop() ;
				}});
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	} ;
    
	// Invariant. (state == State.STOPPED implies currentThread == null) 
	//         and (state == State.RUNNING implies currentThread != null)
    private Thread currentThread = null ;
	private enum State {RUNNING, STOPPED} 
	private State state = State.STOPPED ;
	
    private void startAlgorithm(final Algorithm algorithm) {
    	if( state == State.RUNNING ) return ;
		currentThread  = new Thread( new Runnable() {
			@Override public void run() {
				try {
					algorithm.go(array) ;
				} catch( InterruptedException e ) {
				} catch( Throwable e ) {
					System.err.println( "Unexpected exception " + e.getMessage() ) ;
					e.printStackTrace( System.err) ;
				} finally {
					cleanUp() ; } } }  ) ;
		stopButton.setEnabled( true ) ;
		randomizeButton.setEnabled( false ) ;
		for( JButton button : sortButtons ) 
			button.setEnabled( false ) ;
		state = State.RUNNING ;
		currentThread.start();
	}
    
    private void stopCurrentThread() {
    	if( currentThread != null ) {
    		currentThread.interrupt() ;
    	}
    }
    
    private void stop() {
    	if( state == State.STOPPED ) return ;
    	currentThread = null ;
		stopButton.setEnabled( false ) ;
		randomizeButton.setEnabled( true ) ;
		for( JButton button : sortButtons )
			button.setEnabled( true ) ;
		array.cleanUp () ;
		state = State.STOPPED ;
	}
    

    public Main( ) {
    	randomizeButton = new JButton("Randomize") ;
        algorithmFactory = new AlgorithmFactory() ;
        sortButtons = new JButton[ algorithmFactory.count() ] ;
        stopButton = new JButton("Stop") ;
        
        viewFactory = new ViewFactory() ;
        int count = viewFactory.getViewCount() ;
        viewFrames = new JFrame[ count ] ;
        for( int i = 0 ; i < count ; ++ i ) {
        	viewFrames[i] = new JFrame() ;
        	viewFrames[i].setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE) ;
        	viewFrames[i].setLayout( new BorderLayout() ) ;
        	JPanel view = viewFactory.makeView(i, array) ;
        	viewFrames[i].add( view ) ;
        	viewFrames[i].setLocation(i * 250, 300);
        	viewFrames[i].setSize(200, 200) ;
        	viewFrames[i].setVisible( true ) ;
        }
        
        randomizeButton.addActionListener(
        		new ActionListener() {
        			public void actionPerformed(ActionEvent e) {
        				array.randomize();
            }
        });
        
        for( int i = 0 ; i < sortButtons.length ; ++i ) {
        	final Algorithm algorithm = algorithmFactory.get(i) ;
        	JButton button = new JButton( algorithm.name() ) ;
            button.addActionListener(new ActionListener() {
    			public void actionPerformed(ActionEvent e) {
    				startAlgorithm( algorithm ); } } );
            sortButtons[i] = button ; }
        
        stopButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				stopCurrentThread( ); } } );
        stopButton.setEnabled( false ) ;
        
        mainFrame = new JFrame() ;
        mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE) ;
        mainFrame.setLayout( new GridLayout(2+sortButtons.length, 1, 0, 5) ) ;
        mainFrame.add( randomizeButton ) ;
        for( JButton button : sortButtons ) 
        	mainFrame.add( button ) ;
        mainFrame.add( stopButton ) ;
        mainFrame.setLocation(0,0);
        mainFrame.setSize(200, 200) ;
        mainFrame.setVisible( true ) ;
    }
    
    public static void main( String args[] ) {
    	SwingUtilities.invokeLater( new Runnable () {
    		@Override public void run() {
				new Main() ; }} ) ;
    }
}