package sorting.model;

import sorting.observer.AbstractSubject;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author  Theodore Norvell
 * @version 1.0
 */

public class ObservableArray extends AbstractSubject {

    private int[] dataArray  ;
    public enum DataState { BLACK, RED, BLUE, WHITE ; }
    private DataState[] highlighted ;
    // Invariant dataArray.length == highlighted.length
    private int height ;
    // Invariant 0 <= dataArray[i] && dataArray[i] < height,
    //    for all i s.t. 0 <= i && i < dataArray.length
    
    /**
     * Create an observable array
     * @param length The length of the array
     * @param height The range of values.
     * Precondition: height > 0 ;
     */
    public ObservableArray( int length, int height ) {
        dataArray = new int[length] ;
        highlighted = new DataState[length] ;
        for( int i=0 ; i < length ; ++i ) highlighted[i] = DataState.BLACK ;
        this.height = height ; }
    

    /** Get the maximum size of any item allowed in the array. */
    public synchronized int getHeight() { return height ; }
    
    /** Get the the value at an index.
     * @param idx
     * Precondition: 0 <= idx and idx < length() 
     * */
    public synchronized int get( int idx ) { return dataArray[idx] ; }
    
    /** Get the the highlight colour for the item.
     * @param idx
     * Precondition: 0 <= idx and idx < length() 
     * */
    public synchronized DataState getHighlight( int idx ) { return highlighted[ idx ] ; }
    
    /** Get the the number of items in the array.
     * */
    public synchronized int length() { return dataArray.length ; }
    
    /**
     * Set an item in the array
     * @param idx The item to set
     * @param value. The new value
     * @precondition: 0 <= idx && idx < length()
     * @precondition: 0 <= value && value < getHeight() 
     */
    public synchronized void set( int idx, int value ) {
    	dataArray[idx] = value ;
        notifyObservers() ; }
    /**
     * Set an item in the array
     * @param idx The item to set
     * @param value. The new data state
     * @precondition: 0 <= idx && idx < length()
     */
    public synchronized void setHighlight( int idx, DataState value ) {
        highlighted[idx] = value ;
        notifyObservers() ; }
    
    /**Set the values in the array to random values.
     * <p>
     * The values will uniformly
     * distributed among integers greater than 0 and less than getHeight().
     */
    public synchronized void randomize() {
		for( int i=0 ; i < length() ; ++i ) {
			int r = 1 + (int)( height * Math.random() );
			dataArray[i] = r ; ; } 
        notifyObservers() ;
    }

    /** Set the data stat of all items to DataState.BLACK.
     */
	public synchronized void cleanUp() {
		for( int i = 0 ; i < dataArray.length ; ++i )
			highlighted[i] = DataState.BLACK ;
        notifyObservers() ;
	}
}