package sorting.observer;

import java.util.* ;


/**
 * <p>Title: AbstractSubject</p>
 * <p>Description: Deal with a list of observers. 
 * The observers are notified asynchronously on the event dispatch thread.
 * </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Theodore Norvell
 * @version 1.0
 */

abstract public class AbstractSubject {
    
    private Set<ObserverInterface> observers 
        = new HashSet<ObserverInterface>() ;
    
    /**
     * Attach an observer
     */
    public synchronized void attachObserver( ObserverInterface newObserver ) {
        observers.add( newObserver ) ;
        newObserver.updateObserver() ; }
    
    /**
     * Remove an observer
     */
    public synchronized void dettachObserver( ObserverInterface observer ) {
        observers.remove( observer ) ; }
    
    /**
     * Notify all observers.
     */
    protected synchronized void notifyObservers() {
    	for( final ObserverInterface observer : observers ) {
    		observer.updateObserver() ;
    	}
    }
}