package sorting.views;

import java.awt.LayoutManager;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import sorting.model.ObservableArray;
import sorting.observer.ObserverInterface;

public abstract class AbsPanelViewOfArray extends JPanel
                                          implements ObserverInterface {

	protected ObservableArray array;

	public AbsPanelViewOfArray( ObservableArray array ) {
        this.array = array ;
        array.attachObserver( this ); 
	}

	public AbsPanelViewOfArray(LayoutManager layout) {
		super(layout);
	}

	public AbsPanelViewOfArray(boolean isDoubleBuffered) {
		super(isDoubleBuffered);
	}

	protected abstract Runnable getUpdater();

	public AbsPanelViewOfArray(LayoutManager layout, boolean isDoubleBuffered) {
		super(layout, isDoubleBuffered);
	}

	public void updateObserver() {
		// Since this notification might come on a thread that
		// is not the GUI thread, we do the actual work later on the
		// the GUI thread.
		// Note that this is a somewhat risky because the observable
		// object's state might change concurrently with the execution
		// of the Runnable below. That means that it might get an
		// inconsistent view of the state.  In this particular case,
		// it is not a concern because it doesn't really matter if the
		// use sees an inconsistent visualization and because the 
		// length of the ObservableArray can not change.
		// See my Othello example for a less risky way of doing the
		// observer pattern in multithreaded (and distributed) code.
		SwingUtilities.invokeLater( getUpdater() );
	}

}